// Copyright (c) 2014, The Shelf Route project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

import 'package:shelf/shelf.dart' as shelf;
import 'package:shelf/shelf_io.dart' as io;
import 'package:shelf_route/shelf_route.dart' as route;

void main() {
  var publicRoute = const shelf.Stack()
      .addHandler(_echoRequest);

  var router = (route.router()
      ..addRoute(publicRoute, path: '/public')
      ..addRoute(_bankingRoutes(), path: '/banking'))
      .handler;

  var handler = const shelf.Stack()
      .addMiddleware(shelf.logRequests())
      .addHandler(router);

  io.serve(handler, 'localhost', 8080).then((server) {
    print('Serving at http://${server.address.host}:${server.port}');
  });
}

shelf.Response _echoRequest(shelf.Request request) {
  return new shelf.Response.ok('Request for "${request.pathInfo}"');
}

// supports modularity with routes. i.e. here the banking section has it's own routes
// these are relative to where the main router places them
shelf.Handler _bankingRoutes() {
  var transfersRoute = const shelf.Stack()
      .addHandler(_echoRequest);

  var bankingRouter = (route.router()
      ..addRoute(_accountsRoute(), path: '/account')
      ..addRoute(transfersRoute, path: '/transfer'))
      .handler;

  var bankingRoute = const shelf.Stack()
//      .addMiddleware(authenticator); // obviously they would be authenticated
      .addHandler(bankingRouter);

  return bankingRoute;
}

// all accounts
shelf.Handler _accountsRoute() {
  return (route.router()
      ..addRoute(_individualAccountRoute(), path: '/:accountNumber',
          pathVariables: [route.pathVariable('accountNumber', type: int)])
      ..get('/', _searchAccounts))
      .handler;
}

// a single account with an account number. Note accountNumber is now
// in request.extraParams
shelf.Handler _individualAccountRoute() {
  return (route.router()
      ..get('/', _fetchAccount)
      ..post('/deposit', _deposit))
      .handler;
}

shelf.Response _fetchAccount(shelf.Request request) {
  return new shelf.Response.ok(
      "Fetch account for acct num ${request.headers['shelf.route.accountNumber']}");
}

shelf.Response _deposit(shelf.Request request) {
  return new shelf.Response.ok(
      "Deposit into account for acct num ${request.headers['shelf.route.accountNumber']}");
}

shelf.Response _searchAccounts(shelf.Request request) {
  return new shelf.Response.ok("Search accounts");
}
